##################### generated by xml-casa (v2) from msuvbinflag.xml ###############
##################### 98ef26c0660306926ac8c8cbc6ae416b ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor
_pc = _val_ctor( )
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_msuvbinflag import msuvbinflag as _msuvbinflag_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log

class _msuvbinflag:
    """
    msuvbinflag ---- An algorithm to identify outliers in the UV plane through defined uniform grids by msuvbin.
    Both msuvbin and msuvbinflag tasks are experimental
    

    
    Msuvbinflag is an automatic flagging algorithm for the identification of Radio Frequency Interference (RFI)
    in the UV plane. The visibilities in input CASA Measurements Set (MS) has to be gridded into an uniformed UV plane
    with the msuvbin task.
    The principle undelying this task is that any interference that has effects on a large scale in the image domain will appear
    as a compact structure in UV-domain. No astronomical source will have that behavior especially not centred around (U=0, V=0).
    
    Multiple algorithms will be provided to locate these UV compact structures.
    
    

    --------- parameter descriptions ---------------------------------------------

    binnedvis    Name of input uniform binned visibility data (MS) set on UV plane using msuvbin task.
                 default: none;
                 example: binnedvis='SNR_Binnedvis.ms'
    method       There are four algorithm options proposed for selecting the outliers as flags (radial_per_plane, radial_mean_annular,  regionalMean, gradient)
                  
                  method='radial_per_plane': This algorithm works independently
                  on each plane of the binned ms. A mean radial profile is
                  gereated by doing a weighted average of all data at a given
                  UV-radius. A polynomial fit is done of this radial averaged
                  profile. All the uv points which have amplitude above the this
                  fitted profile at the radius of the uv-point by nsigma is
                  marked as flagged (originalValue[u,v] >
                  radiafitValue[r]+nsigma*rms).
                 
                 method='radial_mean_annular': The di Then average values are
                 calculated for each annular circle with one unit equal spacing.
                 A binary linear least square fitting is generated followed by
                 the comparison analysis between the original value and the
                 fitting value. A point is treated as an outliers to be flagged
                 if original value is greater than the sum of the fitting value
                 and the multiple of the user defined sigma and the interpolated
                 value.  eg: originalValue[x,y] >
                 radiafitValue[r]+sigma*interpolateValue[r].
                 
                 method='regionalMean': This algorithm is to compute the
                 statistical mean, median and standard deviation for every point
                 as a center within a square region which the size is defined by
                 user. A point is treated as an outliers to be flagged if it is
                 beyond the certain criteria threshold.  eg: originalValue[x,y])
                 > regionalMean+sigma*regionalStd
                 
                 method='gradient': This algorithm is to compute the gradient for
                 every point.  Then a point is treated as an outliers to be
                 flagged if there is a sudden change of gradient sign over a few
                 pixel in x and y directions.
                                
                 default: 'radial_per_plane'
                 example: method='radial_mean_annular'
    nsigma       User defined sigma value for comparison for radia algorithm. It is double type.
                 The default value is 5.
    sizeRegion   User defined local square regional size for "regionalMean" and "gradient" algorithm option.
    gradientflip User defined radius value for gradient algorithm. It is double type.
                 The default value is 1.
    doplot       Generate debugging plots; can slow down a lot.
                 The default value is False.
    [1;42mRETURNS[1;m         void

    --------- examples -----------------------------------------------------------

    
    
    
    msuvbinflag(binnedvis='SNR_G55_10s.ms', method='radial_per_plane', nsigma=5)
    
    Keyword arguments:
    binnedvis -- Name of input binned visibility file which is output from msuvbin.
    After msuvbinflag, binnedvis is modified with flagging info.
    
    method -- Three method options: "radian", "regional mean","median","gradient"
    default: 'radian'
    
    nsigma -- User set nsigma value for radial algorithm.
    default=5.0
    
    


    """

    _info_group_ = """flagging, manipulation, imaging"""
    _info_desc_ = """An algorithm to identify outliers in the UV plane through defined uniform grids by msuvbin.
      Both msuvbin and msuvbinflag tasks are experimental
  """

    def __call__( self, binnedvis, method='radial_per_plane', nsigma=float(5.0), sizeRegion=int(20), gradientflip=int(1), doplot=False ):
        schema = {'binnedvis': {'type': 'cStr', 'coerce': _coerce.to_str}, 'method': {'type': 'cVariant', 'coerce': [_coerce.to_variant] # <allowed> IS NOT ALLOWED FOR A PARAMETER OF TYPE any
}, 'nsigma': {'type': 'cFloat', 'coerce': _coerce.to_float}, 'sizeRegion': {'type': 'cInt'}, 'gradientflip': {'type': 'cInt'}, 'doplot': {'type': 'cBool'}}
        doc = {'binnedvis': binnedvis, 'method': method, 'nsigma': nsigma, 'sizeRegion': sizeRegion, 'gradientflip': gradientflip, 'doplot': doplot}
        assert _pc.validate(doc,schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log( 'msuvbinflag', [ 'binnedvis=' + repr(_pc.document['binnedvis']), 'method=' + repr(_pc.document['method']), 'nsigma=' + repr(_pc.document['nsigma']), 'sizeRegion=' + repr(_pc.document['sizeRegion']), 'gradientflip=' + repr(_pc.document['gradientflip']), 'doplot=' + repr(_pc.document['doplot']) ] )
        task_result = None
        try:
            task_result = _msuvbinflag_t( _pc.document['binnedvis'], _pc.document['method'], _pc.document['nsigma'], _pc.document['sizeRegion'], _pc.document['gradientflip'], _pc.document['doplot'] )
        except Exception as exc:
            _except_log('msuvbinflag', exc)
            raise
        finally:
            task_result = _end_log( _logging_state_, 'msuvbinflag', task_result )
        return task_result

msuvbinflag = _msuvbinflag( )

